import logging
import signal
import sys
import time
from datetime import timedelta

from timeloop.exceptions import ServiceExit
from timeloop.helpers import service_shutdown
from timeloop.job import Job


class Timeloop():
    def __init__(self):
        self.jobs = []
        logger = logging.getLogger('timeloop')
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logging.INFO)
        formatter = logging.Formatter('[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s')
        ch.setFormatter(formatter)
        logger.addHandler(ch)
        logger.setLevel(logging.INFO)
        self.logger = logger

    def _add_job(self, func, interval, *args, **kwargs):
        j = Job(interval, func, *args, **kwargs)
        self.jobs.append(j)

    def _block_main_thread(self):
        signal.signal(signal.SIGTERM, service_shutdown)
        signal.signal(signal.SIGINT, service_shutdown)

        while True:
            try:
                time.sleep(1)
            except ServiceExit:
                self.stop()
                break

    def _start_jobs(self, block):
        for j in self.jobs:
            j.daemon = not block
            j.start()
            self.logger.info("Registered job {}".format(j.execute))

    def _stop_jobs(self):
        for j in self.jobs:
            self.logger.info("Stopping job {}".format(j.execute))
            j.stop()

    def job(self, interval: timedelta, num_executions: int = Job.UNLIMITED_EXECUTION, initial_delay: timedelta = None):
        if initial_delay is None:
            initial_delay = interval

        def decorator(f):
            self._add_job(f, interval, num_executions=num_executions, initial_delay=initial_delay)
            return f

        return decorator

    def stop(self):
        self._stop_jobs()
        self.logger.info("Timeloop exited.")

    def start(self, block=False):
        self.logger.info("Starting Timeloop..")
        self._start_jobs(block=block)

        self.logger.info("Timeloop now started. Jobs will run based on the interval set")
        if block:
            self._block_main_thread()



# please install python if it is not present in the system
from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
 name='timeloop-ng',
 version='1.0.0',
 packages=['timeloop'],
 license = 'MIT',
 description = 'An elegant way to run period tasks.',
 author = 'Sankalp Jonna, Christoph Becker',
 author_email = 'tuergeist@gmail.com',
 keywords = ['tasks','jobs','periodic task','interval','periodic job', 'flask style', 'decorator'],
 long_description=long_description,
 long_description_content_type="text/markdown",
 url="https://github.com/tuergeist/timeloop",
 include_package_data=True,
)



import datetime
from threading import Thread, Event


class Job(Thread):
    UNLIMITED_EXECUTION = None

    NUM_EXEC_KEYWORD = "num_executions"
    INITIAL_DELAY_KEYWORD = "initial_delay"

    def __init__(self, interval: datetime.timedelta, execute, *args, **kwargs):
        Thread.__init__(self)
        self.stopped = Event()
        self.interval = interval
        self.execute = execute
        self.args = args
        # to preserve the existing interface, get num_executions from kwargs
        if Job.NUM_EXEC_KEYWORD in kwargs.keys():
            self.num_executions = kwargs.pop(Job.NUM_EXEC_KEYWORD)
        else:
            self.num_executions = Job.UNLIMITED_EXECUTION
        # to preserve the existing interface, get delay_execution from kwargs
        if Job.INITIAL_DELAY_KEYWORD in kwargs.keys():
            self.initial_delay = kwargs.pop(Job.INITIAL_DELAY_KEYWORD)
        else:
            self.initial_delay = interval

        self.kwargs = kwargs
        self.__counter = 0

    def stop(self):
        self.stopped.set()
        self.join()

    def _set_check_counter_and_stop(self):
        self.__counter += 1
        if self.num_executions and self.__counter >= self.num_executions:
            self.stopped.set()

    def run(self):
        if not self.stopped.wait(self.initial_delay.total_seconds()):
            self.execute(*self.args, **self.kwargs)
            self._set_check_counter_and_stop()
        while not self.stopped.wait(self.interval.total_seconds()):
            self.execute(*self.args, **self.kwargs)
            self._set_check_counter_and_stop()